"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var express_1 = tslib_1.__importDefault(require("express"));
var config_1 = require("../config/config");
var network_1 = require("../util/network");
var assets_1 = require("./assets");
var default_html_middleware_1 = require("./middleware/default-html-middleware");
exports.getDefaultHtmlMiddleware = default_html_middleware_1.getDefaultHtmlMiddleware;
var ensure_request_log_middleware_1 = require("./middleware/ensure-request-log-middleware");
var isProduction = process.env.NODE_ENV === 'production';
exports.getPort = function (fallbackPort) {
    var port = parseInt(process.env.PORT || '', 10) || fallbackPort;
    if (!port) {
        throw new Error('No port specified, set one through the PORT environmental variable');
    }
    return port;
};
exports.isWatchMode = function () { return process.env.START_WATCH_MODE === 'true'; };
exports.isFastMode = function () { return process.env.START_FAST_MODE === 'true'; };
exports.createServer = function (options) {
    var config = config_1.getRuntimeConfig(options.log);
    var earlyMiddlewareHook = options.earlyMiddlewareHook, middlewareHook = options.middlewareHook, callback = options.callback, _a = options.startListening, startListening = _a === void 0 ? true : _a;
    // To save the user passing the log to both create server and the SSR middleware
    // we make the log available on the app, so when the app is passed through
    // the ssr middleware can access the logger
    var app = express_1.default();
    app.log = options.log;
    app.disable('x-powered-by');
    // For server hot reload, we need to load default assets
    if (process.env.LOAD_DEFAULT_ASSETS && process.env.PROJECT_DIR) {
        var buildConfig = config_1.getConfig(options.log, process.env.PROJECT_DIR || process.cwd());
        // When running in dev mode, we don't use assets.json so we need to prime
        // the assets location
        assets_1.setDefaultAssets(buildConfig);
    }
    // buildConfig is used for build time config, for example hot reload, and the PORT
    // in local development. In production the port will come from the environment, not
    // this config object
    if (process.env.NODE_ENV !== 'production' && exports.isWatchMode()) {
        var buildConfig = config_1.getConfig(options.log, process.env.PROJECT_DIR || process.cwd());
        // When running in dev mode, we don't use assets.json so we need to prime
        // the assets location
        assets_1.setDefaultAssets(buildConfig);
        // tslint:disable-next-line no-var-requires
        var getHotReloadMiddleware = require('../../server/dev').getHotReloadMiddleware;
        app.use(getHotReloadMiddleware(options.log, buildConfig));
    }
    app.use(ensure_request_log_middleware_1.createEnsureRequestLogMiddleware(options.log));
    if (earlyMiddlewareHook) {
        earlyMiddlewareHook(app);
    }
    // Express route prefixes have to start with /
    var assetsPathPrefixWithLeadingSlash = config.ASSETS_PATH_PREFIX[0] === '/'
        ? config.ASSETS_PATH_PREFIX
        : "/" + config.ASSETS_PATH_PREFIX;
    app.use(assetsPathPrefixWithLeadingSlash, express_1.default.static(config.ASSETS_PATH, {
        index: false,
    }));
    if (config.SERVER_PUBLIC_DIR) {
        app.use(express_1.default.static(config.SERVER_PUBLIC_DIR, {
            index: false,
        }));
    }
    if (middlewareHook) {
        middlewareHook(app);
    }
    // if the server does not use server-side rendering, just respond with index.html
    // for each request not handled in other middlewares
    app.get('*', default_html_middleware_1.getDefaultHtmlMiddleware(options.log, config));
    if (!startListening) {
        return app;
    }
    var listen = function (usePort) {
        var server = app.listen(usePort, function () {
            options.log.info("Server listening on port " + usePort);
            if (process.env.NODE_ENV !== 'production' && exports.isWatchMode()) {
                // tslint:disable-next-line no-var-requires
                var openBrowser = require('../../server/dev').openBrowser;
                openBrowser(usePort);
            }
            if (callback) {
                callback();
            }
        });
        app.set('server', server);
    };
    var port = exports.getPort();
    if (isProduction) {
        listen(port);
    }
    else {
        network_1.findFreePort(port).then(function (usePort) { return listen(usePort); });
    }
    return app;
};
//# sourceMappingURL=server.js.map