"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var fs_1 = tslib_1.__importDefault(require("fs"));
var path_1 = tslib_1.__importDefault(require("path"));
exports.readFileSync = function (log, filePath) {
    try {
        return fs_1.default.readFileSync(filePath, 'utf8');
    }
    catch (err) {
        log.error({ err: err, filePath: filePath }, 'Error reading file');
        throw err;
    }
};
exports.readFile = function (log, filePath) {
    return new Promise(function (resolve, reject) {
        fs_1.default.readFile(filePath, 'utf8', function (err, data) {
            if (err) {
                log.error({ err: err, filePath: filePath }, 'Error reading file');
                reject(err);
            }
            else {
                resolve(data);
            }
        });
    });
};
exports.getFileSize = function (log, filePath) {
    return new Promise(function (resolve, reject) {
        fs_1.default.stat(filePath, function (err, stats) {
            if (err) {
                log.error({ err: err, filePath: filePath }, 'Error getting file size');
                reject(err);
            }
            else {
                resolve(stats.size);
            }
        });
    });
};
exports.writeFile = function (log, filePath, fileContent) {
    return new Promise(function (resolve, reject) {
        fs_1.default.writeFile(filePath, fileContent, function (err) {
            if (err) {
                log.error({ err: err, filePath: filePath }, 'Error writing file');
                reject(err);
            }
            else {
                resolve();
            }
        });
    });
};
exports.formatFileSize = function (size) { return (size / 1024).toFixed(4); };
exports.existsSync = function (file) {
    try {
        fs_1.default.accessSync(file);
        return true;
    }
    catch (ex) {
        return false;
    }
};
/**
 * Dynamically require a file in a way that works both in node and from a webpack bundle
 */
exports.dynamicRequire = function (file) {
    if (typeof __non_webpack_require__ !== 'undefined') {
        return __non_webpack_require__(file);
    }
    else {
        // apparently just wrapping require() in a function is enough
        // to make webpack not try and bundle complete directories.
        // without the presence of above __non_webpack_require__ it would however emit a warning
        // "Critical dependency: the request of a dependency is an expression"
        return require(file);
    }
};
/**
 * Get custon config file if it exists as JavaScript
 * Abort with an error if it only exists as TypeScript
 * @param filePath File path from project root (without extension)
 */
exports.getCustomConfigFile = function (log, root, filePath, fallback) {
    var customConfigFile = path_1.default.resolve(root, filePath + '.js');
    var customConfigFileTS = path_1.default.resolve(root, filePath + '.ts');
    if (exports.existsSync(customConfigFile)) {
        // using dynamicRequire to support bundling project-watchtower with webpack
        var config = exports.dynamicRequire(customConfigFile).default;
        if (typeof config === 'function') {
            return config(root);
        }
        return config;
    }
    // We don't dynamically compile because that would add more runtime
    // dependencies to watchtower.
    if (exports.existsSync(customConfigFileTS)) {
        log.error(filePath + ' only found as TypeScript file.');
        log.error('Please make sure to compile all TS files in the /config folder to JS!');
        process.exit(1);
    }
    return fallback;
};
//# sourceMappingURL=fs.js.map