import * as tslib_1 from "tslib";
import fs from 'fs';
import md5 from 'md5';
import mkdirp from 'mkdirp';
import path from 'path';
import rimraf from 'rimraf';
import { promisify } from 'util';
// Configure Variables
const cacheDir = process.env.BUILD_CACHE_DIRECTORY || `.build-cache`;
const cacheDirPath = path.join(path.resolve(cacheDir));
const tsConfigPath = 'tsconfig.json';
const buildCacheValidationConfig = '.build-cache-validation';
export const TSCONFIG_VALIDATION_ITEM = {
    filePath: tsConfigPath,
    hashKey: 'tsconfigHash',
    isFile: true,
};
// default values
const validatorConfig = {
    baseCacheDirectory: cacheDirPath,
    cacheDirectory: cacheDirPath,
    cacheValidationConfigPath: path.join(cacheDirPath, buildCacheValidationConfig),
    validationItems: [TSCONFIG_VALIDATION_ITEM],
};
let TRACE_MESSAGES = true;
const traceLog = (log, message) => {
    if (TRACE_MESSAGES) {
        log.trace(message);
    }
};
export const setup = (log, options) => {
    const { cacheDirectory, validationItems, valiationConfigPath, traceMessages } = options;
    if (traceMessages !== undefined) {
        TRACE_MESSAGES = traceMessages;
    }
    if (cacheDirectory) {
        validatorConfig.cacheDirectory = cacheDirectory;
        validatorConfig.baseCacheDirectory = cacheDirectory;
    }
    if (validationItems) {
        validatorConfig.validationItems = validationItems;
    }
    if (valiationConfigPath) {
        validatorConfig.cacheValidationConfigPath = valiationConfigPath;
    }
    traceLog(log, `[Cache-Validator] validatorConfig => ${JSON.stringify(validatorConfig)}`);
    return validatorConfig;
};
export const setupWithBuildInfo = (log, options) => {
    const { cacheDirectory, buildInfo, validationItems, traceMessages } = options;
    const { project, environment, target } = buildInfo;
    if (traceMessages !== undefined) {
        TRACE_MESSAGES = traceMessages;
    }
    if (cacheDirectory) {
        validatorConfig.cacheDirectory = cacheDirectory;
        validatorConfig.baseCacheDirectory = cacheDirectory;
    }
    // Project Environment Target Folder
    const petFolder = path.join(project.replace('/', '.'), `${environment}.${target}`);
    const newCacheDir = path.join(validatorConfig.baseCacheDirectory, petFolder);
    // Update config due to petFolder (pet = project environment target)
    validatorConfig.cacheDirectory = newCacheDir;
    validatorConfig.cacheValidationConfigPath = path.join(newCacheDir, '.build-cache-validation');
    if (validationItems) {
        validatorConfig.validationItems = validationItems;
    }
    traceLog(log, `[Cache-Validator] validatorConfig => ${JSON.stringify(validatorConfig)}`);
    return validatorConfig;
};
export const buildCacheDirectory = (buildInfo) => {
    const { project, environment, target } = buildInfo;
    // Project Environment Target Folder
    const petFolder = path.join(project.replace('/', '.'), `${environment}.${target}`);
    return path.join(validatorConfig.baseCacheDirectory, petFolder);
};
const writeFile = promisify(fs.writeFile);
const rmrf = promisify(rimraf);
const fileExists = promisify(fs.exists);
const getFileContents = (log, filePath) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const readFile = promisify(fs.readFile);
    const exists = yield fileExists(filePath);
    if (exists) {
        const contents = yield readFile(filePath, { encoding: 'utf8' });
        traceLog(log, `[Cache-Validator] contents from ${filePath} is '${contents}'`);
        return contents;
    }
    traceLog(log, `[Cache-Validator] ${filePath} does not exist, no contents returned`);
    return null;
});
export const getConfigContents = (log, configFilePath) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const result = yield getFileContents(log, configFilePath);
    if (result) {
        const cast = JSON.parse(result);
        return cast;
    }
    return null;
});
const writeContents = (log, filePath, contents) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const jsonStr = JSON.stringify(contents);
    traceLog(log, `[Cache-Validator] saving cacheLoaderValidation =>${jsonStr}`);
    const exists = yield fileExists(filePath);
    if (!exists) {
        const parentDir = path.dirname(filePath);
        mkdirp.sync(parentDir);
    }
    yield writeFile(filePath, jsonStr, { encoding: 'utf8', flag: 'w+' });
});
export const deletePath = (log, fileOrDir) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    traceLog(log, `[Cache-Validator] rmrf ${fileOrDir}`);
    yield rmrf(fileOrDir);
});
const clearDirectory = (log, directory) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const exists = yield fileExists(directory);
    if (exists) {
        log.info(`[Cache-Validator] Clearing => ${directory}`);
    }
    else {
        log.info(`[Cache-Validator] ${directory} not cleared (Does not exist)`);
    }
    yield deletePath(log, directory);
});
const getMd5OfFile = (log, key, file) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const tsConfig = yield getFileContents(log, file);
    if (tsConfig) {
        const md5Hash = md5(tsConfig);
        traceLog(log, `[Cache-Validator] md5 hash for file => ${key} contents -> ${md5Hash}`);
        return md5Hash;
    }
    return null;
});
export const getMd5 = (log, key, item) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const md5Hash = md5(item);
    log.info(`[Cache-Validator] md5 hash for ${key} -> ${md5Hash}`);
    return md5Hash;
});
const shouldClearCache = (log) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const cacheLoaderValidation = yield getConfigContents(log, validatorConfig.cacheValidationConfigPath);
    if (cacheLoaderValidation) {
        const config = cacheLoaderValidation;
        if (validatorConfig.validationItems.length > 0) {
            let result = false;
            yield Promise.all(validatorConfig.validationItems.map((validationItem) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                traceLog(log, `[Cache-Validator] processing validation Item ${JSON.stringify(validationItem)}`);
                const hash = validationItem.isFile
                    ? yield getMd5OfFile(log, validationItem.hashKey, validationItem.filePath)
                    : validationItem.itemHash;
                if (hash !== config[validationItem.hashKey]) {
                    log.info(`[Cache-Validator] ${validationItem.hashKey} hashMismatch, config=${config[validationItem.hashKey]} ${validationItem.isFile ? 'file' : 'object'}=${hash}`);
                    result = true;
                }
            })));
            if (!result) {
                log.info(`[Cache-Validator] No mismatches found - cache not cleared`);
                config.cacheCleared = false;
                yield writeContents(log, validatorConfig.cacheValidationConfigPath, config);
            }
            return result;
        }
    }
    traceLog(log, `[Cache-Validator] ${validatorConfig.cacheValidationConfigPath} not found (Does not exist)`);
    return true;
});
const createValidationConfig = (log, cacheCleared) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const configToWrite = { cacheCleared };
    yield Promise.all(validatorConfig.validationItems.map((validationItem) => tslib_1.__awaiter(this, void 0, void 0, function* () {
        const hashValue = validationItem.isFile
            ? yield getMd5OfFile(log, validationItem.hashKey, validationItem.filePath)
            : validationItem.itemHash;
        if (hashValue) {
            configToWrite[validationItem.hashKey] = hashValue;
        }
    })));
    log.info(`[Cache-Validator] writing config file => ${validatorConfig.cacheValidationConfigPath}`);
    yield writeContents(log, validatorConfig.cacheValidationConfigPath, configToWrite);
});
/**
 * Determines if the build cache should be cleared if the ts config has changed.
 */
export const validateCache = (log) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    log.info(`[Cache-Validator] cacheDirectory => ${validatorConfig.cacheDirectory}`);
    const clear = yield shouldClearCache(log);
    if (clear) {
        yield clearDirectory(log, validatorConfig.cacheDirectory);
        yield createValidationConfig(log, true);
    }
});
export const writeDummyConfigFile = (log, file, config) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const contents = JSON.stringify(config);
    traceLog(log, `[Cache-Validator] writing dummy config ${file} with contents ${contents}`);
    yield writeFile(file, contents, 'utf8');
});
export const writeDummyFile = (log, file, contents) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    traceLog(log, `[Cache-Validator] writing dummy file ${file} with contents ${contents}`);
    yield writeFile(file, contents, 'utf8');
});
//# sourceMappingURL=cache-validator.js.map