import * as tslib_1 from "tslib";
import dotenv from 'dotenv';
import http from 'http';
import os from 'os';
import path from 'path';
import { getPort } from '../runtime/server/server';
import { findFreePort, waitForConnection } from '../runtime/util/network';
import { forkPromise } from '../util/process';
import { getTimeMs, timeout } from '../util/time';
const getIp = () => {
    const interfaces = os.networkInterfaces();
    for (const i of Object.keys(interfaces)) {
        for (const address of interfaces[i]) {
            if (address.family === 'IPv4' && !address.internal) {
                return address.address;
            }
        }
    }
    return os.hostname();
};
const getServerUrl = (port, urlPath) => {
    const useUrlPath = urlPath.indexOf('/') === 0 ? urlPath : '/' + urlPath;
    const host = process.env.STATS_SERVER_ADDRESS || getIp() || 'localhost'; // provided by build environment, ref OPS-383
    return `http://${host}:${port}${useUrlPath}`;
};
export const loadSSRPage = (url) => new Promise((resolve, reject) => {
    const startTime = getTimeMs();
    const request = http.get(url, res => {
        res.setEncoding('utf8');
        let size = 0;
        let content = '';
        res.on('data', chunk => {
            size += chunk.length;
            content += chunk.toString();
        });
        res.on('end', () => {
            const time = getTimeMs() - startTime;
            resolve({ size, time, content });
        });
    });
    request.on('error', err => reject(err));
});
export const runStatsOnServer = (log, buildConfig, statsFn, verbose = false) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const { OUTPUT, HAS_SERVER, STATS_ENV, STATS_PAGES } = buildConfig;
    if (!HAS_SERVER) {
        log.info('Skipping server-based stats because the application has no server');
        return;
    }
    dotenv.config({
        path: path.join(buildConfig.BASE, '.env'),
    });
    const port = yield findFreePort(getPort(buildConfig.DEV_SERVER_PORT));
    const serverEntryFile = path.resolve(OUTPUT, 'server.js');
    const devServer = yield forkPromise(log, serverEntryFile, [], {
        env: Object.assign({ NODE_ENV: 'production' }, process.env, STATS_ENV, { PORT: port, PROJECT_DIR: buildConfig.BASE }),
        silent: !verbose,
    }, true);
    try {
        yield timeout(waitForConnection(port), 20000);
        for (const page in STATS_PAGES) {
            if (STATS_PAGES.hasOwnProperty(page)) {
                const urlPath = STATS_PAGES[page];
                const url = getServerUrl(port, urlPath);
                // warm-up
                for (let i = 0; i < 3; i++) {
                    yield timeout(loadSSRPage(url), 20000);
                }
                yield statsFn({ page, urlPath, url, port });
            }
        }
    }
    catch (err) {
        throw err;
    }
    finally {
        devServer.kill();
    }
});
//# sourceMappingURL=server.js.map