/**
 * PromiseCompletionSource allows the resolution/rejection of a promise to be
 * controled outside of the promise ctor callback
 *
 * Example:
 * const pcs = new PromiseCompletionSource<any>()
 *
 * const promise = pcs.promise
 * pcs.resolve('data')
 */
var PromiseCompletionSource = /** @class */ (function () {
    function PromiseCompletionSource(autoReset) {
        if (autoReset === void 0) { autoReset = false; }
        var _this = this;
        this.autoReset = autoReset;
        this.completed = false;
        this.resolve = function () {
            return Promise.reject('Nothing is observing this PromiseCompletionSource, cannot resolve');
        };
        this.reject = function () {
            return Promise.reject('Nothing is observing this PromiseCompletionSource, cannot reject');
        };
        this.reset = function () {
            _this.completed = false;
            _this._promise = new Promise(function (resolve, reject) {
                _this.resolve = function (result) {
                    _this.ensureNotComplete();
                    resolve(result);
                    return new Promise(function (r) { return setTimeout(r); });
                };
                _this.reject = function (error) {
                    _this.ensureNotComplete();
                    reject(error);
                    // We want to resolve this promise
                    // because we are using this to know when callbacks are resolved
                    // not get the error, you can await pcs.promise to get error
                    return new Promise(function (r) { return setTimeout(r); });
                };
            });
            return _this._promise;
        };
        this.ensureNotComplete = function () {
            if (_this.completed) {
                throw new Error('Promise already completed');
            }
            _this.completed = true;
        };
    }
    Object.defineProperty(PromiseCompletionSource.prototype, "promise", {
        get: function () {
            if (!this.autoReset && this._promise) {
                return this._promise;
            }
            if (this._promise) {
                if (!this.completed) {
                    // tslint:disable-next-line:max-line-length
                    this.reject(new Error('Cannot get new promise when autoReset is true and existing promise was not resolved'));
                }
            }
            return this.reset();
        },
        enumerable: true,
        configurable: true
    });
    return PromiseCompletionSource;
}());
export { PromiseCompletionSource };
//# sourceMappingURL=promise-completion-source.js.map